"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var Uglify = require("uglify-js");
var logger_1 = require("./logger/logger");
var config_1 = require("./util/config");
var errors_1 = require("./util/errors");
var helpers_1 = require("./util/helpers");
var worker_client_1 = require("./worker-client");
function uglifyjs(context, configFile) {
    configFile = config_1.getUserConfigFile(context, exports.taskInfo, configFile);
    var logger = new logger_1.Logger('uglifyjs');
    return worker_client_1.runWorker('uglifyjs', 'uglifyjsWorker', context, configFile)
        .then(function () {
        logger.finish();
    })
        .catch(function (err) {
        throw logger.fail(new errors_1.BuildError(err));
    });
}
exports.uglifyjs = uglifyjs;
function uglifyjsWorker(context, configFile) {
    var uglifyJsConfig = config_1.fillConfigDefaults(configFile, exports.taskInfo.defaultConfigFile);
    if (!context) {
        context = config_1.generateContext(context);
    }
    return uglifyjsWorkerImpl(context, uglifyJsConfig);
}
exports.uglifyjsWorker = uglifyjsWorker;
function uglifyjsWorkerImpl(context, uglifyJsConfig) {
    return __awaiter(this, void 0, void 0, function () {
        var jsFilePaths, promises, ex_1, errorString;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _a.trys.push([0, 2, , 3]);
                    jsFilePaths = context.bundledFilePaths.filter(function (bundledFilePath) { return bundledFilePath.endsWith('.js'); });
                    promises = jsFilePaths.map(function (filePath) {
                        var sourceMapPath = filePath + '.map';
                        return runUglifyInternal(filePath, filePath, sourceMapPath, sourceMapPath, uglifyJsConfig);
                    });
                    return [4 /*yield*/, Promise.all(promises)];
                case 1: return [2 /*return*/, _a.sent()];
                case 2:
                    ex_1 = _a.sent();
                    errorString = ex_1.message + " in " + ex_1.filename + " at line " + ex_1.line + ", col " + ex_1.col + ", pos " + ex_1.pos;
                    throw new errors_1.BuildError(new Error(errorString));
                case 3: return [2 /*return*/];
            }
        });
    });
}
exports.uglifyjsWorkerImpl = uglifyjsWorkerImpl;
function runUglifyInternal(sourceFilePath, destFilePath, sourceMapPath, destMapPath, configObject) {
    return __awaiter(this, void 0, void 0, function () {
        var sourceFileContentPromise, _a, sourceFileContent, sourceMapContent, uglifyConfig, result;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    sourceFileContentPromise = helpers_1.readFileAsync(sourceFilePath);
                    return [4 /*yield*/, Promise.all([helpers_1.readFileAsync(sourceFilePath), helpers_1.readFileAsync(sourceMapPath)])];
                case 1:
                    _a = _b.sent(), sourceFileContent = _a[0], sourceMapContent = _a[1];
                    uglifyConfig = Object.assign({}, configObject, {
                        sourceMap: {
                            content: sourceMapContent
                        }
                    });
                    result = Uglify.minify(sourceFileContent, uglifyConfig);
                    if (result.error) {
                        throw new errors_1.BuildError("Uglify failed: " + result.error.message);
                    }
                    return [2 /*return*/, Promise.all([helpers_1.writeFileAsync(destFilePath, result.code), helpers_1.writeFileAsync(destMapPath, result.map)])];
            }
        });
    });
}
exports.taskInfo = {
    fullArg: '--uglifyjs',
    shortArg: '-u',
    envVar: 'IONIC_UGLIFYJS',
    packageConfig: 'ionic_uglifyjs',
    defaultConfigFile: 'uglifyjs.config'
};
